import { Method, MockConfigData, MockItemExt, RequestInfo } from '../types';
import MockItem from './mock-item';
export default class Mocker {
    private static instance;
    private mockConfigData;
    private disabled;
    private log;
    constructor();
    static getInstance(): Mocker;
    /**
     * Set global mock data configuration.
     * @param {object} mockConfigData
     */
    setMockData(mockConfigData: MockConfigData): this;
    /**
     * Add an mock item to global mock data configuration.
     * @param {string} key
     * @param {any} val
     */
    private addMockItem;
    /**
     * Reset global mock data configuration.
     * @param {string} key
     * @param {any} val
     */
    reset(): this;
    /**
     * Enable mock function temporarily.
     */
    enable(): this;
    /**
     * Disable mock function temporarily.
     */
    disable(): this;
    /**
     * Disable logs function temporarily.
     */
    disableLog(): this;
    /**
     * Disable logs function temporarily.
     */
    enableLog(): this;
    /**
     * Check specified mock item & add it to global mock data configuration.
     * @param {MockItem} mockItem
     * @returns false | MockItem
     */
    mock(mockItemInfo: MockItem): false | MockItem;
    /**
     * Make a mock item that matches an HTTP GET request.
     * @param {RegExp | String} url
     * @param {any} body
     * @param {MockItemExt} opts {
     *    @param {number} delay
     *    @param {number} status
     *    @param {object} header
     *    @param {number} times
     * }
     */
    get(url: RegExp | String, body: any, opts?: MockItemExt): this;
    /**
     * Make a mock item that matches an HTTP POST request.
     * @param {RegExp | String} url
     * @param {any} body
     * @param {MockItemExt} opts {
     *    @param {number} delay
     *    @param {number} status
     *    @param {object} header
     *    @param {number} times
     * }
     */
    post(url: RegExp | String, body: any, opts?: MockItemExt): this;
    /**
     * Make a mock item that matches an HTTP PUT request.
     * @param {RegExp | String} url
     * @param {any} body
     * @param {MockItemExt} opts {
     *    @param {number} delay
     *    @param {number} status
     *    @param {object} header
     *    @param {number} times
     * }
     */
    put(url: RegExp | String, body: any, opts?: MockItemExt): this;
    /**
     * Make a mock item that matches an HTTP PATCH request.
     * @param {RegExp | String} url
     * @param {any} body
     * @param {MockItemExt} opts {
     *    @param {number} delay
     *    @param {number} status
     *    @param {object} header
     *    @param {number} times
     * }
     */
    patch(url: RegExp | String, body: any, opts?: MockItemExt): this;
    /**
     * Make a mock item that matches an HTTP DELETE request.
     * @param {RegExp | String} url
     * @param {any} body
     * @param {MockItemExt} opts {
     *    @param {number} delay
     *    @param {number} status
     *    @param {object} header
     *    @param {number} times
     * }
     */
    delete(url: RegExp | String, body: any, opts?: MockItemExt): this;
    /**
     * https://developer.mozilla.org/en-US/docs/Web/HTTP/Methods/HEAD
     * Warning: A response to a HEAD method should not have a body.
     * If it has one anyway, that body must be ignored: any representation
     * headers that might describe the erroneous body are instead assumed
     * to describe the response which a similar GET request would have received.
     *
     * Make a mock item that matches an HTTP HEAD request.
     * @param {RegExp | String} url
     * @param {MockItemExt} opts {
     *    @param {number} delay
     *    @param {number} status
     *    @param {object} header
     *    @param {number} times
     * }
     */
    head(url: RegExp | String, opts?: MockItemExt): this;
    /**
     * Make a mock item that matches an HTTP GET, POST, PUT, PATCH, DELETE or HEAD request.
     * @param {RegExp | String} url
     * @param {any} body
     * @param {MockItemExt} opts {
     *    @param {number} delay
     *    @param {number} status
     *    @param {object} header
     *    @param {number} times
     * }
     */
    any(url: RegExp | String, body: any, opts?: MockItemExt): this;
    /**
     * Check whether the specified request url matchs a defined mock item.
     * If a match is found, return the matched mock item, otherwise a null is returned.
     * @param {string} reqUrl
     * @param {string} reqMethod
     * @return null | MockItem
     */
    matchMockItem(reqUrl: string, reqMethod: Method | undefined): MockItem | null;
    groupLog(logs: any[]): void;
    sendResponseLog(spent: number, body: any, requestInfo: RequestInfo, mockItem: MockItem): void;
}
